/* Sistemas Integrados TECNISA
   Planejamento Alimentao Industrial

Nome: CUSTO_PRODUTO_ALMOX.SQL

Descrio:
Retorna o custo de um Produto em relao a um almoxarifado em uma determinada data.

P_CDFILIAL     - Cdigo da Filial.
P_CDPRODUTO    - Cdigo do Produto.
P_NRTIPOCUST   - Nmero do Tipo de Custo (1 a 10)
P_DTREFERENCIA - Data do Custo.

TIPOS DE CUSTO:

01 Custo Mdio Lquido Atual
02 Custo Mdio Bruto Atual
03 Custo Mdio Lquido da poca
04 Custo Mdio Bruto da poca
06 Custo de Cotao (Lista de Preos)
07 Custo da ltima compra(AF)
08 Custo Lquido da poca (ltima Compra)
09 Custo Bruto da poca (ltima Compra)
10 Custo de Cotao(Lista de Preos com Imposto)
20 Custo Mdio Lquido Atual
21 Custo Mdio Lquido Atual
22 Custo Mdio Bruto Atual
23 Custo Mdio Bruto Atual
24 Custo Mdio Lquido da poca
25 Custo Mdio Lquido da poca
26 Custo Mdio Bruto da poca
27 Custo Mdio Bruto da poca
28 Custo da ltima Compra (NF)
29 Custo da ltima Compra (NF)
30 Custo de Cotao (Lista de Preos)
31 Custo de Cotao (Lista de Preos)
32 Custo da ltima Compra(AF)
33 Custo da ltima Compra(AF)
34 Custo Mdio Atualizado

40 Custo Mdio Lquido Atual(Logstica)
41 Custo Mdio Bruto Atual(Logstica)
42 Custo Mdio Lquido da poca(Logstica)
43 Custo Mdio Bruto da poca(Logstica)
44 Custo de Cotao (Lista de Preos - Logstica)
45 Custo da ltima compra(AF-Logstica)
46 Custo de Compra Liquido da poca(Logstica)
47 Custo de Compra Bruto da poca(Logstica)
48 Custo de Cotao-Lista de Preo Imposto(Logstica)
49 Custo Mdio Lquido Atual(Logstca)
50 Custo Mdio Lquido Atual(Logstica)
51 Custo Mdio Bruto Atual(Logstica)
52 Custo Mdio Bruto Atual(Logstica)
53 Custo Mdio Lquido da poca(Logstica)
54 Custo Mdio Lquido da poca(Logstica)
55 Custo Mdio Bruto da poca(Logstica)
56 Custo Mdio Bruto da poca(Logstica)
57 Custo da ltima Compra (NF - Logstica)
58 Custo da ltima Compra (NF - Logstica)
59 Custo da ltima Compra(AF - Logstica)
60 Custo da ltima Compra(AF - Logstica)
61 Custo de Cotao (Lista de Preos - Logstica)
62 Custo de Cotao (Lista de Preos - Logstica)

@C:\Web_649\Sql\End_Conv\Oracle\0318_CUSFUNO_CUSTO_PRODUTO_ALMOX.SQL

Criao:
19/01/2024 - Jeferson E.R Arajo

Manutenes:
__/__/____ - _______ ___________
__/__/____ - _______ ___________
__/__/____ - _______ ___________
__/__/____ - _______ ___________
*/

create or replace FUNCTION CUSTO_PRODUTO_ALMOX ( 
   P_CDFILIAL     IN FILIAL.CDFILIAL%TYPE,
   P_CDPRODUTO    IN PRODUTO.CDPRODUTO%TYPE,
   P_NRTIPOCUSTO  IN NUMBER,
   P_DTREFERENCIA IN DATE,
   P_DTATUAL      IN DATE,
   P_ALMOXARIFE   IN ALMOXARIFADO.CDALMOXARIFE%TYPE DEFAULT ' ',
   P_LOCALESTOQ   IN LOCALESTOQUE.CDLOCALESTOQ%TYPE DEFAULT ' '
) RETURN FLOAT
IS
   V_VRCUSTO      POSESTDIA.VRESTOQDIA%TYPE;
   V_SEM_IMPOSTO  POSESTDIA.VRESTOQDIA%TYPE;
   V_DTREFERENCIA DATE;
   V_DTREFERENCIA_MAIS_UM DATE;
   V_QTESTOQDIA     POSESTDIA.QTESTOQDIA%TYPE;
   V_QTESTOQANT     POSESTDIA.QTESTOQANT%TYPE;
   V_VRESTOQDIA     POSESTDIA.VRESTOQDIA%TYPE;
   V_VRESTOQDIAM1   POSESTDIA.VRESTOQDIAM1%TYPE;
   V_VRESTOQDIAM2   POSESTDIA.VRESTOQDIAM2%TYPE;
   V_VRESTOQANT     POSESTDIA.VRESTOQANT%TYPE;
   V_VRESTOQANTM1   POSESTDIA.VRESTOQANTM1%TYPE;
   V_VRESTOQANTM2   POSESTDIA.VRESTOQANTM2%TYPE;
   V_VRESTOQBRUT    POSESTDIA.VRESTOQBRUT%TYPE;
   V_VRESTOQBRUM1   POSESTDIA.VRESTOQBRUM1%TYPE;
   V_VRESTOQBRUM2   POSESTDIA.VRESTOQBRUM2%TYPE;
   V_VRBRUTOANT     POSESTDIA.VRBRUTOANT%TYPE;
   V_VRBRUTOANTM1   POSESTDIA.VRBRUTOANTM1%TYPE;
   V_VRBRUTOANTM2   POSESTDIA.VRBRUTOANTM2%TYPE;
   V_VRCUSPRLISTP   CUSTOSUP.VRCUSPRLISTP%TYPE;
   V_VRCUSPRLISM1   CUSTOSUP.VRCUSPRLISM1%TYPE;
   V_VRCUSPRLISM2   CUSTOSUP.VRCUSPRLISM2%TYPE;
   V_VRCUSLIPRIMP   CUSTOSUP.VRCUSLIPRIMP%TYPE;
   V_VRPRCUSTOAF    CUSTOSUP.VRPRCUSTOAF%TYPE;
   V_VRPRCUSAFM1    CUSTOSUP.VRPRCUSAFM1%TYPE;
   V_VRPRCUSAFM2    CUSTOSUP.VRPRCUSAFM2%TYPE;
   V_IDUTILDTPROC   PARAMGERAL.IDUTILDTPROC%TYPE;
   V_VRCLOGDIA      POSESTDIA.VRCLOGDIA%TYPE;
   V_VRCLOGDIAANT   POSESTDIA.VRCLOGDIAANT%TYPE;
   V_VRCLOGDIAM1    POSESTDIA.VRCLOGDIAM1%TYPE;
   V_VRCLOGDIAAM1   POSESTDIA.VRCLOGDIAAM1%TYPE;
   V_VRCLOGDIAM2    POSESTDIA.VRCLOGDIAM2%TYPE;
   V_VRCLOGDIAAM2   POSESTDIA.VRCLOGDIAAM2%TYPE;
   V_VRSEGUDIA      POSESTDIA.VRSEGUDIA%TYPE;
   V_VRSEGUDIAANT   POSESTDIA.VRSEGUDIAANT%TYPE;
   V_VRSEGUDIAM1    POSESTDIA.VRSEGUDIAM1%TYPE;
   V_VRSEGUDIAAM1   POSESTDIA.VRSEGUDIAAM1%TYPE;
   V_VRSEGUDIAM2    POSESTDIA.VRSEGUDIAM2%TYPE;
   V_VRSEGUDIAAM2   POSESTDIA.VRSEGUDIAAM2%TYPE;
   V_CDPRODUTO      POSESTDIA.CDPRODUTO%TYPE;
   V_CDPRODACOR     CUSTOSUP.CDPRODACOR%TYPE;
   V_CDPRODLIST     CUSTOSUP.CDPRODLIST%TYPE;
   V_FORNECEDOR     FORNECEDOR.CDFORNECED%TYPE;
   V_CDFORNECACOR   FORNECEDOR.CDFORNECED%TYPE;
   V_CDFORNECLIST   FORNECEDOR.CDFORNECED%TYPE;
   V_CDFORNEOL      FORNECEDOR.CDFORNECED%TYPE;
   V_VRPESOBRUTPR   PRODUTO.VRPESOBRUTPR%TYPE;
   V_VRPECUSTOLOG   POSESTDIA.VRCLOGDIA%TYPE;
   V_VRVRCUSTOLOG   POSESTDIA.VRCLOGDIA%TYPE;
   V_VRPESEGURLOG   POSESTDIA.VRCLOGDIAM1%TYPE;
   V_VRCUSTOLOG     POSESTDIA.VRCLOGDIA%TYPE;
   V_VRCUSTOSEG     POSESTDIA.VRSEGUDIA%TYPE;
   V_VRLIQCMPPROD   CUSTOCMPPROD.VRLIQCMPPROD%TYPE;
   V_VRBRUCMPPROD   CUSTOCMPPROD.VRBRUCMPPROD%TYPE;
   V_VRLIQCMPPRM1   CUSTOCMPPROD.VRLIQCMPPRM1%TYPE;
   V_VRLIQCMPPRM2   CUSTOCMPPROD.VRLIQCMPPRM2%TYPE;
   V_VRCUSTLOGCMP   CUSTOCMPPROD.VRCUSTLOGCMP%TYPE;
   V_VRCUSTLCMPM1   CUSTOCMPPROD.VRCUSTLCMPM1%TYPE;
   V_VRCUSTLCMPM2   CUSTOCMPPROD.VRCUSTLCMPM2%TYPE;
   V_VRSEGULOGCMP   CUSTOCMPPROD.VRSEGULOGCMP%TYPE;
   V_VRSEGULCMPM1   CUSTOCMPPROD.VRSEGULCMPM1%TYPE;
   V_VRSEGULCMPM2   CUSTOCMPPROD.VRSEGULCMPM2%TYPE;
   V_CDFORNCUSTAF   FORNECEDOR.CDFORNECED%TYPE;
   V_CDPRODCUSTAF   PRODUTO.CDPRODUTO%TYPE;
   V_CDFILIAL       FILIAL.CDFILIAL%TYPE;
   V_VRFATOCONV     PRODUTO.VRFATOCONV%TYPE;

   W_N              CHAR(1) := 'N';
   W_0001           CHAR(4) := '0001';
   V_ZERO           FLOAT := 0;
   V_UM             FLOAT := 1;

BEGIN
   V_VRCUSTO      := 0;
   V_VRCUSTOLOG   := 0;
   V_VRCUSTOSEG   := 0;
   V_VRPESOBRUTPR := 0;
   V_VRPECUSTOLOG := 0;
   V_VRVRCUSTOLOG := 0;
   V_VRPESEGURLOG := 0;
   V_VRCUSTLOGCMP := 0;
   V_VRCUSTLCMPM1 := 0;
   V_VRCUSTLCMPM2 := 0;
   V_VRSEGULOGCMP := 0;
   V_VRSEGULCMPM1 := 0;
   V_VRSEGULCMPM2 := 0;

   IF P_NRTIPOCUSTO IN (1,2,3,4,20,21,22,23,24,25,26,27,40,41,42,43,49,50,53,54) THEN /*Custos que tem valores na tabela POSESTDIA*/

      IF (P_NRTIPOCUSTO IN (1,2,20,21,22,23,40,41,49,50,51,52)) THEN /*Custo Mdio Atual*/
         BEGIN
            SELECT NVL(IDUTILDTPROC, W_N)
            INTO V_IDUTILDTPROC
            FROM PARAMGERAL
            WHERE CDPARGERAL = W_0001;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_IDUTILDTPROC := W_N;
         END;

         IF V_IDUTILDTPROC = W_N THEN
            V_DTREFERENCIA := P_DTATUAL;
         ELSE
            BEGIN
               SELECT NVL(DTPROCESSA, P_DTATUAL)
               INTO V_DTREFERENCIA
               FROM PARAMFILIAL P
               WHERE CDFILIAL = P_CDFILIAL;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_DTREFERENCIA := P_DTATUAL;
            END;
         END IF;
      ELSE
         V_DTREFERENCIA := P_DTREFERENCIA;/*Custo Mdio da poca*/
      END IF;
      V_DTREFERENCIA_MAIS_UM:= V_DTREFERENCIA+1;

      /*OK*/
      BEGIN
         SELECT NVL(SUM(QTESTOQDIA),V_ZERO),   NVL(SUM(QTESTOQANT),V_ZERO),NVL(SUM(VRESTOQDIA),V_ZERO),
                NVL(SUM(VRESTOQDIAM1),V_ZERO), NVL(SUM(VRESTOQDIAM2),V_ZERO),NVL(SUM(VRESTOQANT),V_ZERO),
                NVL(SUM(VRESTOQANTM1),V_ZERO), NVL(SUM(VRESTOQANTM2),V_ZERO),NVL(SUM(VRESTOQBRUT),V_ZERO),
                NVL(SUM(VRESTOQBRUM1),V_ZERO), NVL(SUM(VRESTOQBRUM2),V_ZERO),NVL(SUM(VRBRUTOANT),V_ZERO),
                NVL(SUM(VRBRUTOANTM1),V_ZERO), NVL(SUM(VRBRUTOANTM2),V_ZERO),NVL(SUM(VRCLOGDIA),V_ZERO),
                NVL(SUM(VRCLOGDIAANT),V_ZERO), NVL(SUM(VRCLOGDIAM1),V_ZERO),NVL(SUM(VRCLOGDIAAM1),V_ZERO),
                NVL(SUM(VRCLOGDIAM2 ),V_ZERO), NVL(SUM(VRCLOGDIAAM2),V_ZERO),NVL(SUM(VRSEGUDIA),V_ZERO),
                NVL(SUM(VRSEGUDIAANT),V_ZERO), NVL(SUM(VRSEGUDIAM1),V_ZERO),NVL(SUM(VRSEGUDIAAM1),V_ZERO),
                NVL(SUM(VRSEGUDIAM2),V_ZERO),  NVL(SUM(VRSEGUDIAAM2),V_ZERO)
         INTO V_QTESTOQDIA, V_QTESTOQANT, V_VRESTOQDIA,
              V_VRESTOQDIAM1, V_VRESTOQDIAM2, V_VRESTOQANT,
              V_VRESTOQANTM1, V_VRESTOQANTM2, V_VRESTOQBRUT,
              V_VRESTOQBRUM1, V_VRESTOQBRUM2, V_VRBRUTOANT,
              V_VRBRUTOANTM1, V_VRBRUTOANTM2, V_VRCLOGDIA,
              V_VRCLOGDIAANT, V_VRCLOGDIAM1,  V_VRCLOGDIAAM1,
              V_VRCLOGDIAM2,  V_VRCLOGDIAAM2, V_VRSEGUDIA,
              V_VRSEGUDIAANT, V_VRSEGUDIAM1,  V_VRSEGUDIAAM1,
              V_VRSEGUDIAM2, V_VRSEGUDIAAM2
         FROM POSESTDIA C
         WHERE C.CDFILIAL       = P_CDFILIAL   AND
               C.CDALMOXARIFE   = P_ALMOXARIFE AND
               C.CDLOCALESTOQ   = P_LOCALESTOQ AND
               C.CDPRODUTO      = P_CDPRODUTO  AND
               (C.DTPOSIESTQ   = (SELECT MAX(DTPOSIESTQ) DTPOSIESTQ
                                    FROM POSESTDIA A
                                   WHERE CDFILIAL       = P_CDFILIAL
                                     AND CDALMOXARIFE   = P_ALMOXARIFE
                                     AND CDLOCALESTOQ   = P_LOCALESTOQ
                                     AND CDPRODUTO      = P_CDPRODUTO
                                     AND DTPOSIESTQ     <= V_DTREFERENCIA_MAIS_UM
                                     and A.NRLOTEESTQ = C.NRLOTEESTQ
                                     and A.NRSUBLOTE = C.NRSUBLOTE));
      EXCEPTION
         WHEN NO_DATA_FOUND THEN BEGIN
            V_QTESTOQDIA   := V_ZERO;  V_QTESTOQANT   := V_ZERO;  V_VRESTOQDIA   := V_ZERO;
            V_VRESTOQDIAM1 := V_ZERO;  V_VRESTOQDIAM2 := V_ZERO;  V_VRESTOQANT   := V_ZERO;
            V_VRESTOQANTM1 := V_ZERO;  V_VRESTOQANTM2 := V_ZERO;  V_VRESTOQBRUT  := V_ZERO;
            V_VRESTOQBRUM1 := V_ZERO;  V_VRESTOQBRUM2 := V_ZERO;  V_VRBRUTOANT   := V_ZERO;
            V_VRBRUTOANTM1 := V_ZERO;  V_VRBRUTOANTM2 := V_ZERO;  V_VRCLOGDIA    := V_ZERO;
            V_VRCLOGDIAANT := V_ZERO;  V_VRCLOGDIAM1  := V_ZERO;  V_VRCLOGDIAAM1 := V_ZERO;
            V_VRCLOGDIAM2  := V_ZERO;  V_VRCLOGDIAAM2 := V_ZERO;  V_VRSEGUDIA    := V_ZERO;
            V_VRSEGUDIAANT := V_ZERO;  V_VRSEGUDIAM1  := V_ZERO;  V_VRSEGUDIAAM1 := V_ZERO;
            V_VRSEGUDIAM2  := V_ZERO;  V_VRSEGUDIAAM2 := V_ZERO;
         END;
      END;

      IF (V_QTESTOQDIA <> 0) THEN
         V_VRESTOQDIA   := (V_VRESTOQDIA/V_QTESTOQDIA);
         V_VRESTOQDIAM1 := (V_VRESTOQDIAM1/V_QTESTOQDIA);
         V_VRESTOQDIAM2 := (V_VRESTOQDIAM2/V_QTESTOQDIA);
         V_VRESTOQBRUT  := (V_VRESTOQBRUT/V_QTESTOQDIA);
         V_VRESTOQBRUM1 := (V_VRESTOQBRUM1/V_QTESTOQDIA);
         V_VRESTOQBRUM2 := (V_VRESTOQBRUM2/V_QTESTOQDIA);

         V_VRCLOGDIA    := (V_VRCLOGDIA/V_QTESTOQDIA);
         V_VRCLOGDIAM1  := (V_VRCLOGDIAM1/V_QTESTOQDIA);
         V_VRCLOGDIAM2  := (V_VRCLOGDIAM2/V_QTESTOQDIA);
         V_VRSEGUDIA    := (V_VRSEGUDIA/V_QTESTOQDIA);
         V_VRSEGUDIAM1  := (V_VRSEGUDIAM1/V_QTESTOQDIA);
         V_VRSEGUDIAM2  := (V_VRSEGUDIAM2/V_QTESTOQDIA);

      ELSIF (V_QTESTOQANT <> 0) THEN
         V_VRESTOQDIA   := (V_VRESTOQANT/V_QTESTOQANT);
         V_VRESTOQDIAM1 := (V_VRESTOQANTM1/V_QTESTOQANT);
         V_VRESTOQDIAM2 := (V_VRESTOQANTM2/V_QTESTOQANT);
         V_VRESTOQBRUT  := (V_VRBRUTOANT/V_QTESTOQANT);
         V_VRESTOQBRUM1 := (V_VRBRUTOANTM1/V_QTESTOQANT);
         V_VRESTOQBRUM2 := (V_VRBRUTOANTM2/V_QTESTOQANT);

         V_VRCLOGDIA     := (V_VRCLOGDIAANT/V_QTESTOQANT);
         V_VRCLOGDIAM1   := (V_VRCLOGDIAAM1/V_QTESTOQANT);
         V_VRCLOGDIAM2   := (V_VRCLOGDIAAM2/V_QTESTOQANT);
         V_VRSEGUDIA     := (V_VRSEGUDIAANT/V_QTESTOQANT);
         V_VRSEGUDIAM1   := (V_VRSEGUDIAAM1/V_QTESTOQANT);
         V_VRSEGUDIAM2   := (V_VRSEGUDIAAM2/V_QTESTOQANT);
      ELSE
         V_VRESTOQDIA   := 0;
         V_VRESTOQDIAM1 := 0;
         V_VRESTOQDIAM2 := 0;
         V_VRESTOQBRUT  := 0;
         V_VRESTOQBRUM1 := 0;
         V_VRESTOQBRUM2 := 0;

         V_VRCLOGDIA    := 0;
         V_VRCLOGDIAM1  := 0;
         V_VRCLOGDIAM2  := 0;
         V_VRSEGUDIA    := 0;
         V_VRSEGUDIAM1  := 0;
         V_VRSEGUDIAM2  := 0;
      END IF;

      IF P_NRTIPOCUSTO IN (1,3,40,42) THEN      /*Custo Mdio Lquido*/
         V_VRCUSTO := V_VRESTOQDIA;

         IF P_NRTIPOCUSTO IN (40,42) THEN  /*Custo Mdio Lquido(Logistico)*/
            V_VRCUSTO := V_VRCUSTO + V_VRCLOGDIA + V_VRSEGUDIA;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (2,4,41,43) THEN   /*Custo Mdio Bruto*/
         V_VRCUSTO := V_VRESTOQBRUT;

         IF P_NRTIPOCUSTO IN (41,43) THEN  /*Custo Mdio Bruto(Logistico)*/
            V_VRCUSTO := V_VRCUSTO + V_VRCLOGDIA + V_VRSEGUDIA;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (20,24,49,53) THEN /*Custo Mdio Lquido (ndice 01)*/
         V_VRCUSTO := V_VRESTOQDIAM1;

         IF P_NRTIPOCUSTO IN (49,53) THEN  /*Custo Mdio Lquido(Logistico) Indice 01*/
            V_VRCUSTO := V_VRCUSTO + V_VRCLOGDIAM1+V_VRSEGUDIAM1;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (21,25,50,54) THEN /*Custo Mdio Lquido (ndice 02)*/
         V_VRCUSTO := V_VRESTOQDIAM2;

         IF P_NRTIPOCUSTO IN (40,54) THEN  /*Custo Mdio Lquido(Logistico) Indice 02*/
            V_VRCUSTO := V_VRCUSTO + V_VRCLOGDIAM2+V_VRSEGUDIAM2;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (22,26,51,55) THEN /*Custo Mdio Bruto (ndice 01)*/
         V_VRCUSTO := V_VRESTOQBRUM1;

         IF P_NRTIPOCUSTO IN (51,55) THEN  /*Custo Mdio Bruto(Logistico) Indice 01*/
            V_VRCUSTO := V_VRCUSTO + V_VRCLOGDIAM1+V_VRSEGUDIAM1;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (23,27,52,56) THEN /*Custo Mdio Bruto (ndice 02)*/
         V_VRCUSTO := V_VRESTOQBRUM2;

         IF P_NRTIPOCUSTO IN (52,56) THEN  /*Custo Mdio Bruto(Logistico) Indice 02*/
            V_VRCUSTO := V_VRCUSTO + V_VRCLOGDIAM2+V_VRSEGUDIAM2;
         END IF;
      END IF;

   ELSIF P_NRTIPOCUSTO IN (6,7,10,30,31,32,33,44,48,45,59,60,61,62) THEN /*Custos que tem valores na tabela CUSTOSUP*/
      V_CDFORNEOL := NULL;
      IF P_NRTIPOCUSTO IN (44, 45, 48, 59, 60,61,62) THEN
         SELECT
            MAX(P.CDFILIOPELOG)
         INTO
            V_CDFILIAL
         FROM
            PARAMPRODSUP P
         WHERE
            P.CDFILIAL  = P_CDFILIAL AND
            P.CDPRODUTO = P_CDPRODUTO;

         IF V_CDFILIAL IS NULL THEN
            V_CDFILIAL := P_CDFILIAL;
         ELSE
            BEGIN
               SELECT
                  CDFORNOPELOG
               INTO
                  V_CDFORNEOL
               FROM
                  FILIAL
               WHERE
                  FILIAL.CDFILIAL = V_CDFILIAL;
            EXCEPTION
               WHEN NO_DATA_FOUND THEN
                  V_CDFORNEOL := NULL;
            END;
         END IF;
      ELSE
         V_CDFILIAL := P_CDFILIAL;
      END IF;

      IF P_NRTIPOCUSTO IN (6, 10, 30, 31, 44, 48, 61, 62) THEN
         BEGIN
            SELECT NVL(VRCUSPRLISTP,V_ZERO), NVL(VRCUSPRLISM1,V_ZERO), NVL(VRCUSPRLISM2,V_ZERO),
                   NVL(VRCUSLIPRIMP,V_ZERO), CDPRODACOR,               CDPRODLIST,
                   CDFORNECLIST,             CDFORNECACOR
            INTO V_VRCUSPRLISTP, V_VRCUSPRLISM1, V_VRCUSPRLISM2,
                 V_VRCUSLIPRIMP, V_CDPRODACOR,   V_CDPRODLIST,
                 V_CDFORNECLIST, V_CDFORNECACOR
            FROM CUSTOSUP
            WHERE CDFILIAL   = V_CDFILIAL  AND
                  CDPRODUTO  = P_CDPRODUTO AND
                  DTCUSTOSUP = (SELECT MAX(DTCUSTOSUP)
                                FROM CUSTOSUP
                                WHERE CDFILIAL  = V_CDFILIAL  AND
                                      CDPRODUTO = P_CDPRODUTO AND
                                      VRCUSPRLISTP IS NOT NULL AND
                                      DTCUSTOSUP <= P_DTREFERENCIA);
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_VRCUSPRLISTP := V_ZERO;  V_VRCUSPRLISM1 := V_ZERO;  V_VRCUSPRLISM2 := V_ZERO;
               V_VRCUSLIPRIMP := V_ZERO;  V_CDPRODACOR   := NULL;    V_CDPRODLIST   := NULL;
               V_CDFORNECLIST := NULL;    V_CDFORNECACOR := NULL;
         END;
      END IF;

      IF P_NRTIPOCUSTO IN (7, 32, 33, 45, 59, 60) THEN
         BEGIN
            SELECT NVL(VRPRCUSTOAF,V_ZERO), NVL(VRPRCUSAFM1,V_ZERO), NVL(VRPRCUSAFM2,V_ZERO),
                   CDFORNCUSTAF, CDPRODCUSTAF
            INTO V_VRPRCUSTOAF, V_VRPRCUSAFM1, V_VRPRCUSAFM2, V_CDFORNCUSTAF,
                 V_CDPRODCUSTAF
            FROM CUSTOSUP
            WHERE CDFILIAL   = V_CDFILIAL  AND
                  CDPRODUTO  = P_CDPRODUTO AND
                  DTCUSTOSUP = (SELECT MAX(DTCUSTOSUP)
                                FROM CUSTOSUP
                                WHERE CDFILIAL  = V_CDFILIAL  AND
                                      CDPRODUTO = P_CDPRODUTO AND
                                      VRPRCUSTOAF IS NOT NULL AND
                                      DTCUSTOSUP <= P_DTREFERENCIA);
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_VRPRCUSTOAF := V_ZERO;  V_VRPRCUSAFM1 := V_ZERO;  V_VRPRCUSAFM2 := V_ZERO;  V_CDFORNCUSTAF := NULL;
               V_CDPRODCUSTAF := NULL;
         END;
      END IF;

/*      Lista de preo e cotao(Logistico)    [06,10][44,48]*/

      IF (V_CDPRODACOR IS NOT NULL) THEN
         V_CDPRODUTO   := V_CDPRODACOR;
         V_FORNECEDOR  := V_CDFORNECACOR;
      END IF;

      IF (V_CDPRODLIST IS NOT NULL) THEN
         V_CDPRODUTO    := V_CDPRODLIST;
         V_FORNECEDOR   := V_CDFORNECLIST;
      END IF;

      IF (V_CDPRODCUSTAF IS NOT NULL) THEN
         V_CDPRODUTO    := V_CDPRODCUSTAF;
         V_FORNECEDOR   := V_CDFORNCUSTAF;
      END IF;

      IF P_NRTIPOCUSTO IN (44, 45, 48, 59, 60,61,62) THEN
         BEGIN
            SELECT
               NVL(VRFATOCONV, V_UM)
            INTO
               V_VRFATOCONV
            FROM
               PRODUTO
            WHERE
               PRODUTO.CDPRODUTO = V_CDPRODUTO;
         EXCEPTION
            WHEN NO_DATA_FOUND THEN
               V_VRFATOCONV := V_UM;
         END;

         SELECT
            NVL(MAX(X.VRPESOBRUTPR),V_ZERO),
            NVL(MAX(VRPECUSTOLOG),V_ZERO),
            NVL(MAX(VRVRCUSTOLOG),V_ZERO),
            NVL(MAX(VRPESEGURLOG),V_ZERO)
         INTO
            V_VRPESOBRUTPR,
            V_VRPECUSTOLOG,
            V_VRVRCUSTOLOG,
            V_VRPESEGURLOG
         FROM
            PRODUTO P,
            CUSTOLOGIST C,
            (SELECT
                NVL(MAX(A.VRPESOBRUTPR),V_ZERO) VRPESOBRUTPR,
                   MAX(P.CDARVPROD) CDPRODUTO,
                   MAX(C.DTINIVIGCLOG) DTINIVIGCLOG
             FROM
                PRODUTO P,
                CUSTOLOGIST C,
                PRODUTO A
             WHERE
                (INSTR(A.CDARVPROD,P.CDARVPROD) = V_UM)         AND
                A.CDPRODUTO    = V_CDPRODUTO                    AND
                P.CDPRODUTO    = C.CDPRODUTO                    AND
                C.CDFILIAL     = P_CDFILIAL                     AND
                C.CDFORNECED   = NVL(V_CDFORNEOL, V_FORNECEDOR) AND
                C.DTINIVIGCLOG = (SELECT MAX(DTINIVIGCLOG)
                                  FROM CUSTOLOGIST A, PRODUTO B, PRODUTO Q
                                  WHERE
                                     A.CDFILIAL     = C.CDFILIAL              AND
                                     A.CDFORNECED   = C.CDFORNECED            AND
                                     (INSTR(B.CDARVPROD,Q.CDARVPROD) = V_UM)  AND
                                     B.CDPRODUTO    = V_CDPRODUTO             AND
                                     Q.CDPRODUTO    = A.CDPRODUTO             AND
                                     A.DTINIVIGCLOG <= P_DTREFERENCIA))X
         WHERE C.CDFILIAL     = P_CDFILIAL                     AND
               C.CDPRODUTO    = P.CDPRODUTO                    AND
               C.CDPRODUTO    = X.CDPRODUTO                    AND
               C.CDFORNECED   = NVL(V_CDFORNEOL, V_FORNECEDOR) AND
               C.DTINIVIGCLOG = X.DTINIVIGCLOG;
      END IF;

      IF V_VRFATOCONV IS NULL THEN
         V_VRFATOCONV := 1;
      END IF;
/***********************************/

      IF P_NRTIPOCUSTO IN (6,44) THEN           /*Custo de Cotao (Lista de Preos)*/

         V_VRCUSTO := V_VRCUSPRLISTP;

         IF NVL(V_VRCUSTO, 0) > 0 THEN
            IF P_NRTIPOCUSTO = 44  THEN
               IF (V_VRVRCUSTOLOG > 0) AND (V_VRPESOBRUTPR > 0) THEN
                  V_VRCUSTOLOG := (V_VRPESOBRUTPR * V_VRVRCUSTOLOG)/V_VRFATOCONV;
               END IF;

               IF (V_VRPECUSTOLOG > 0) THEN
                  V_VRCUSTOLOG := (V_VRCUSTO * V_VRPECUSTOLOG)/100;
               END IF;

               IF (V_VRPESEGURLOG > 0) THEN
                  V_VRCUSTOSEG := (V_VRCUSTO * V_VRPESEGURLOG)/100;
               END IF;

               V_VRCUSTO := V_VRCUSTO + NVL(V_VRCUSTOLOG, 0) + NVL(V_VRCUSTOSEG, 0);
            END IF;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (7,45) THEN   /*Custo da ltima compra(AF)*/
         V_VRCUSTO := V_VRPRCUSTOAF;

         IF NVL(V_VRCUSTO, 0) > 0 THEN
            IF P_NRTIPOCUSTO = 45  THEN
               IF (V_VRVRCUSTOLOG > 0) AND (V_VRPESOBRUTPR > 0) THEN
                  V_VRCUSTOLOG := (V_VRPESOBRUTPR * V_VRVRCUSTOLOG)/V_VRFATOCONV;
               END IF;

               IF (V_VRPECUSTOLOG > 0) THEN
                  V_VRCUSTOLOG := (V_VRCUSTO * V_VRPECUSTOLOG)/100;
               END IF;

               IF (V_VRPESEGURLOG > 0) THEN
                  V_VRCUSTOSEG := (V_VRCUSTO * V_VRPESEGURLOG)/100;
               END IF;

               V_VRCUSTO := V_VRCUSTO + NVL(V_VRCUSTOLOG, 0) + NVL(V_VRCUSTOSEG, 0);
            END IF;
         END IF;
      ELSIF P_NRTIPOCUSTO IN (10,48) THEN        /*Custo de Cotao(Lista de Preos com Imposto)*/

         V_VRCUSTO     := V_VRCUSLIPRIMP;
         V_SEM_IMPOSTO := V_VRCUSPRLISTP;

         IF NVL(V_VRCUSTO, 0) > 0 THEN
            IF P_NRTIPOCUSTO = 48  THEN
               IF (V_VRVRCUSTOLOG > 0) AND (V_VRPESOBRUTPR > 0) THEN
                  V_VRCUSTOLOG := (V_VRPESOBRUTPR * V_VRVRCUSTOLOG)/V_VRFATOCONV;
               END IF;

               IF (V_VRPECUSTOLOG > 0) THEN
                  V_VRCUSTOLOG := (V_VRCUSTO * V_VRPECUSTOLOG)/100;
               END IF;

               IF (V_VRPESEGURLOG > 0) THEN
                  V_VRCUSTOSEG := (V_SEM_IMPOSTO * V_VRPESEGURLOG)/100;
               END IF;

               V_VRCUSTO := V_VRCUSTO + NVL(V_VRCUSTOLOG, 0) + NVL(V_VRCUSTOSEG, 0);
            END IF;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (30, 61) THEN       /*Custo de Cotao (Lista de Preos) (ndice 01)*/
         V_VRCUSTO := V_VRCUSPRLISM1;

         IF NVL(V_VRCUSTO, 0) > 0 THEN
            IF P_NRTIPOCUSTO = 61  THEN
               IF (V_VRVRCUSTOLOG > 0) AND (V_VRPESOBRUTPR > 0) THEN
                  V_VRCUSTOLOG := (V_VRPESOBRUTPR * V_VRVRCUSTOLOG)/V_VRFATOCONV;
               END IF;

               IF (V_VRPECUSTOLOG > 0) THEN
                  V_VRCUSTOLOG := (V_VRCUSTO * V_VRPECUSTOLOG)/100;
               END IF;

               IF (V_VRPESEGURLOG > 0) THEN
                  V_VRCUSTOSEG := (V_VRCUSTO * V_VRPESEGURLOG)/100;
               END IF;

               V_VRCUSTO := V_VRCUSTO + NVL(V_VRCUSTOLOG, 0) + NVL(V_VRCUSTOSEG, 0);
            END IF;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (31, 62) THEN       /*Custo de Cotao (Lista de Preos) (ndice 02)*/
         V_VRCUSTO := V_VRCUSPRLISM2;

         IF NVL(V_VRCUSTO, 0) > 0 THEN
            IF P_NRTIPOCUSTO = 62  THEN
               IF (V_VRVRCUSTOLOG > 0) AND (V_VRPESOBRUTPR > 0) THEN
                  V_VRCUSTOLOG := (V_VRPESOBRUTPR * V_VRVRCUSTOLOG)/V_VRFATOCONV;
               END IF;

               IF (V_VRPECUSTOLOG > 0) THEN
                  V_VRCUSTOLOG := (V_VRCUSTO * V_VRPECUSTOLOG)/100;
               END IF;

               IF (V_VRPESEGURLOG > 0) THEN
                  V_VRCUSTOSEG := (V_VRCUSTO * V_VRPESEGURLOG)/100;
               END IF;

               V_VRCUSTO := V_VRCUSTO + NVL(V_VRCUSTOLOG, 0) + NVL(V_VRCUSTOSEG, 0);
            END IF;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (32,59) THEN       /*Custo da ltima Compra(AF) (ndice 01)*/
         V_VRCUSTO := V_VRPRCUSAFM1;
         IF NVL(V_VRCUSTO, 0) > 0 THEN
            IF P_NRTIPOCUSTO = 59  THEN
               IF (V_VRVRCUSTOLOG > 0) AND (V_VRPESOBRUTPR > 0) THEN
                  V_VRCUSTOLOG := (V_VRPESOBRUTPR * V_VRVRCUSTOLOG)/V_VRFATOCONV;
               END IF;

               IF (V_VRPECUSTOLOG > 0) THEN
                  V_VRCUSTOLOG := (V_VRCUSTO * V_VRPECUSTOLOG)/100;
               END IF;

               IF (V_VRPESEGURLOG > 0) THEN
                  V_VRCUSTOSEG := (V_VRCUSTO * V_VRPESEGURLOG)/100;
               END IF;

               V_VRCUSTO := V_VRCUSTO + NVL(V_VRCUSTOLOG, 0) + NVL(V_VRCUSTOSEG, 0);
            END IF;
         END IF;
      ELSIF P_NRTIPOCUSTO IN (33,60) THEN       /*Custo da ltima Compra(AF) (ndice 02)*/
         V_VRCUSTO := V_VRPRCUSAFM2;
         IF NVL(V_VRCUSTO, 0) > 0 THEN
            IF P_NRTIPOCUSTO = 60  THEN
               IF (V_VRVRCUSTOLOG > 0) AND (V_VRPESOBRUTPR > 0) THEN
                  V_VRCUSTOLOG := (V_VRPESOBRUTPR * V_VRVRCUSTOLOG)/V_VRFATOCONV;
               END IF;

               IF (V_VRPECUSTOLOG > 0) THEN
                  V_VRCUSTOLOG := (V_VRCUSTO * V_VRPECUSTOLOG)/100;
               END IF;

               IF (V_VRPESEGURLOG > 0) THEN
                  V_VRCUSTOSEG := (V_VRCUSTO * V_VRPESEGURLOG)/100;
               END IF;

               V_VRCUSTO := V_VRCUSTO + NVL(V_VRCUSTOLOG, 0) + NVL(V_VRCUSTOSEG, 0);
            END IF;
         END IF;
      END IF;

   ELSIF P_NRTIPOCUSTO IN (8,9,28,29,46,47) THEN /*Custos que tem valores na tabela POSESTDIA*/
      BEGIN
         SELECT NVL(SUM(VRLIQCMPPROD),V_ZERO), NVL(SUM(VRBRUCMPPROD),V_ZERO), NVL(SUM(VRLIQCMPPRM1),V_ZERO),
                NVL(SUM(VRLIQCMPPRM2),V_ZERO), NVL(SUM(VRCUSTLOGCMP),V_ZERO), NVL(SUM(VRCUSTLCMPM1),V_ZERO),
                NVL(SUM(VRCUSTLCMPM2),V_ZERO), NVL(SUM(VRSEGULOGCMP),V_ZERO), NVL(SUM(VRSEGULCMPM1),V_ZERO),
                NVL(SUM(VRSEGULCMPM2),V_ZERO)
         INTO  V_VRLIQCMPPROD, V_VRBRUCMPPROD, V_VRLIQCMPPRM1, V_VRLIQCMPPRM2, V_VRCUSTLOGCMP,
               V_VRCUSTLCMPM1, V_VRCUSTLCMPM2, V_VRSEGULOGCMP, V_VRSEGULCMPM1, V_VRSEGULCMPM2
         FROM CUSTOCMPPROD
         WHERE CDFILIAL       = P_CDFILIAL       AND
               CDALMOXARIFE   = P_ALMOXARIFE     AND
               CDLOCALESTOQ   = P_LOCALESTOQ     AND
               CDPRODUTO      = P_CDPRODUTO      AND
               DTCUSTCMPPRO   =(SELECT MAX(C.DTCUSTCMPPRO)
                                   FROM CUSTOCMPPROD C
                                   WHERE C.CDFILIAL       = P_CDFILIAL   AND
                                         C.CDPRODUTO      = P_CDPRODUTO  AND
                                         C.CDALMOXARIFE   = P_ALMOXARIFE AND
                                         C.CDLOCALESTOQ   = P_LOCALESTOQ AND
                                         C.DTCUSTCMPPRO   <= P_DTREFERENCIA);
      EXCEPTION
         WHEN NO_DATA_FOUND THEN
            V_VRLIQCMPPROD := V_ZERO;  V_VRBRUCMPPROD := V_ZERO;  V_VRLIQCMPPRM1 := V_ZERO;  V_VRLIQCMPPRM2 := V_ZERO;  V_VRCUSTLOGCMP := V_ZERO;
            V_VRCUSTLCMPM1 := V_ZERO;  V_VRCUSTLCMPM2 := V_ZERO;  V_VRSEGULOGCMP := V_ZERO;  V_VRSEGULCMPM1 := V_ZERO;  V_VRSEGULCMPM2 := V_ZERO;
      END;

      IF P_NRTIPOCUSTO IN (8,46) THEN
         V_VRCUSTO := V_VRLIQCMPPROD;
         IF P_NRTIPOCUSTO = 46 THEN
            V_VRCUSTO := V_VRCUSTO + V_VRCUSTLOGCMP + V_VRSEGULOGCMP;
         END IF;
      ELSIF P_NRTIPOCUSTO IN (9,47) THEN
         V_VRCUSTO := V_VRBRUCMPPROD;
         IF P_NRTIPOCUSTO = (47) THEN
            V_VRCUSTO := V_VRCUSTO + V_VRCUSTLOGCMP + V_VRSEGULOGCMP;
         END IF;
      ELSIF P_NRTIPOCUSTO IN (28,50) THEN

         V_VRCUSTO := V_VRLIQCMPPRM1;
         IF P_NRTIPOCUSTO = (50) THEN
            V_VRCUSTO := V_VRCUSTO + V_VRCUSTLCMPM1 + V_VRSEGULCMPM1;
         END IF;

      ELSIF P_NRTIPOCUSTO IN (29,51) THEN
         V_VRCUSTO := V_VRLIQCMPPRM2;

         IF P_NRTIPOCUSTO = (51) THEN
            V_VRCUSTO := V_VRCUSTO + V_VRCUSTLCMPM2 + V_VRSEGULCMPM2;
         END IF;
      END IF;
   ELSIF P_NRTIPOCUSTO = 63 THEN
      SELECT NVL(MAX(C.VRCUSTOCTBL),0)
        INTO V_VRCUSTO
        FROM CUSTMEDCTBL C, DATA D
       WHERE C.CDFILIAL = P_CDFILIAL
         AND C.CDPRODUTO = P_CDPRODUTO
         AND D.DTDATA = P_DTREFERENCIA
         AND C.DTCOMPCTBL = D.DTMES;
   END IF;
   RETURN(V_VRCUSTO);
END;
/
-- VERSAO --
BEGIN ATUVEROBJ('CUSTO_PRODUTO_ALMOX','6.49.xxx'); END;
/

